/* *********************************************************************** *
 * Copyright (C) 2009 by Ulrich Van Den Hekke                              *
 * ulrich.vdh@shadoware.org                                                *
 *                                                                         *
 * This program is free software: you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
 * *********************************************************************** */

// Expat Parser
#include <expat.h>

// Qt header
#include <QString>
#include <QTest>

static void XMLCALL startElement(void *userData, const char *, const char **) {
	int * depthPtr = (int*)userData;
	*depthPtr += 1;
}

static void XMLCALL endElement(void *userData, const char *) {
	int *depthPtr = (int *)userData;
	* depthPtr -= 1;
}

void parseWithExpatParser( const QString & filename ) {
	FILE * file = fopen( qPrintable( filename ), "r" );
	if( file ) {
		char buf[BUFSIZ];
		XML_Parser parser = XML_ParserCreate( NULL );
		int done, depth = 0;
		XML_SetUserData( parser, &depth );
		XML_SetElementHandler( parser, startElement, endElement );
		do {
			int len = (int)fread( buf, 1, sizeof(buf), file );
			done    = len < sizeof(buf);
			if( XML_Parse( parser, buf, len, done ) == XML_STATUS_ERROR ) {
				qDebug( "Can't parse the file" );
				XML_ParserFree( parser );
				fclose( file );
				return ;
			}
		} while( !done );
		XML_ParserFree( parser );

		fclose( file );
	}
}
