/* *********************************************************************** *
 * Copyright (C) 2009 by Ulrich Van Den Hekke                              *
 * ulrich.vdh@shadoware.org                                                *
 *                                                                         *
 * This program is free software: you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
 * *********************************************************************** */

// Xmlev Parser
#ifdef XDK10
#include <xmlctx.hpp>
#else
#include <oraxml.hpp>
#endif

// Qt header
#include <QString>
#include <QTest>

// Std unix
#include <sys/types.h>
#include <sys/wait.h>

#ifdef XDK10
static void parse( const char * filename ) {
	CXmlCtx * ctxp = 0;
	try {
		ctxp = new CXmlCtx();
	} catch( XmlException & e ) {
		unsigned ecode = e.getCode();

		QFAIL( qPrintable( QString("Failed to initialize XML context, error %1").arg( ecode ) ) );
	}

	Factory<CXmlCtx,xmlnode> * fp = 0;

	try {
		fp = new Factory<CXmlCtx,xmlnode>( ctxp );
	} catch( FactoryException & fe ) {
		unsigned ecode = fe.getCode();

		QFAIL( qPrintable( QString("Failed to create create parser, error %1").arg( ecode ) ) );
	}

	DOMParser<CXmlCtx,xmlnode> * parserp = 0;

	try {
		parserp = fp->createDOMParser( DOMParCXml, NULL );
	} catch( FactoryException & fe ) {
		delete fp;
		unsigned ecode = fe.getCode();

		QFAIL( qPrintable( QString("Failed to create parser, error %1").arg( ecode ) ) );
	}

	const char * fname = filename;
	FileSource * isrcp = new FileSource( (oratext*) fname );

	try {
		DocumentRef<xmlnode> * docrefp = parserp->parse( isrcp, FALSE );
		if( docrefp == NULL ) {
			QFAIL( qPrintable( QString("NULL document") ) );
			return;
		}
		xmlnode * np = docrefp->getDocumentElement();
		if( np == NULL ) {
			QFAIL( qPrintable( QString("Empty document") ) );
			return;
		}


		docrefp->markToDelete();
		delete docrefp;
	} catch( ParserException & pe ) {
		delete parserp;
		delete isrcp;
		delete fp;

		unsigned ecode = pe.getCode();

		QFAIL( qPrintable( QString( "Failed to parse the document, error %1").arg( ecode ) ) );
	}

	delete parserp;
	delete isrcp;
	delete fp;
//	delete ctxp;
}
#else
static void parse( const char * filename ) {
	XMLParser parser;
	ub4       flags = XML_FLAG_DISCARD_WHITESPACE;
	uword     ecode;

	if( ( ecode = parser.xmlinit() ) ) {
		QFAIL( qPrintable( QString( "Failed to initialize XML parser, error %1" ).arg( ecode ) ) );
	}

	if( ( ecode = parser.xmlparse( (oratext*)  filename, (oratext*)0, flags ) ) ) {
		QFAIL( qPrintable( QString( "Failed to parse XML document, error %1" ).arg( ecode ) ) );
	}

	Node * node = parser.getDocumentElement();
	if( ! node ) {
		QFAIL( "No root document" );
	}

	parser.xmlterm();
}
#endif

void parseWithOracleParser( const QString & filename ) {
	pid_t pid = fork();
	if( pid > 0 ) {
		waitpid( pid, (int*)0, 0 );
	} else if( pid == 0 ) {
		if( ! filename.isEmpty() )
			parse( qPrintable( filename ) );
		exit( 0 );
	} else {
		QFAIL( "Cannot fork" );
	}
}

