/* *********************************************************************** *
 * Copyright (C) 2009 by Ulrich Van Den Hekke                              *
 * ulrich.vdh@shadoware.org                                                *
 *                                                                         *
 * This program is free software: you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
 * *********************************************************************** */

// Scew
#include <scew/scew.h>

// Qt header
#include <QString>
#include <QTest>

void parseWithScewParser( const QString & filename ) {
	scew_tree * tree     = NULL;
	scew_parser * parser = NULL;

	parser = scew_parser_create();

	if( ! scew_parser_load_file( parser, qPrintable( filename ) ) ) {
		QString message;
		scew_error code = scew_error_code();
		message = QString( "Unable to load file (error %1: %2)\n" ).arg( code ).arg( scew_error_string( code ) );
		if( code == scew_error_expat ) {
			enum XML_Error expat_code = scew_error_expat_code(parser);
			message +=  QString( "Expat error %1 (line %2, column %3) : %4\n" ).arg( (int)expat_code ).arg( scew_error_expat_line( parser ) ).arg( scew_error_expat_column( parser ) ).arg( scew_error_expat_string( expat_code ) );
		}
		QFAIL( qPrintable( message ) );
	}

	tree = scew_parser_tree( parser );

	scew_tree_free( tree );

	scew_parser_free( parser );
}
