/* *********************************************************************** *
 * Copyright (C) 2009 by Ulrich Van Den Hekke                              *
 * ulrich.vdh@shadoware.org                                                *
 *                                                                         *
 * This program is free software: you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
 * *********************************************************************** */

// Xerces-C
#include <xercesc/dom/DOMErrorHandler.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/parsers/AbstractDOMParser.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMImplementationRegistry.hpp>
#include <xercesc/dom/DOMLSParser.hpp>
#include <xercesc/dom/DOMException.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMError.hpp>
#include <xercesc/dom/DOMLocator.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMAttr.hpp>

// Qt header
#include <QString>
#include <QTest>

class StrX {
public :
	StrX(const XMLCh* const toTranscode) {
		XERCES_CPP_NAMESPACE_USE
		// Call the private transcoding method
		fLocalForm = XMLString::transcode(toTranscode);
	}

	~StrX() {
		XERCES_CPP_NAMESPACE_USE
		XMLString::release(&fLocalForm);
	}
	const char* localForm() const {
		return fLocalForm;
	}

private :
	char*   fLocalForm;
};

void parseWithXercesCppParser( const QString & filename ) {
	XERCES_CPP_NAMESPACE_USE

	try {
		XMLPlatformUtils::Initialize();

		// Instantiate the DOM parser.
		static const XMLCh gLS[]  = { chLatin_L, chLatin_S, chNull };
		DOMImplementation *impl   = DOMImplementationRegistry::getDOMImplementation( gLS );
		DOMLSParser       *parser = ( (DOMImplementationLS*) impl )->createLSParser( DOMImplementationLS::MODE_SYNCHRONOUS, 0 );
		DOMConfiguration  *config = parser->getDomConfig();

		config->setParameter(XMLUni::fgDOMNamespaces, false);
		config->setParameter(XMLUni::fgXercesSchema, false);
		config->setParameter(XMLUni::fgXercesSchemaFullChecking, false);
		config->setParameter(XMLUni::fgDOMValidateIfSchema, false);
		config->setParameter(XMLUni::fgDOMValidate, false);

		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc = 0;
		parser->resetDocumentPool();
		doc = parser->parseURI( qPrintable(filename) );

		parser->release();
		XMLPlatformUtils::Terminate();
	} catch( XMLException & e ) {
		QFAIL( qPrintable( QString("Error durring initialization or parsing ! : \n%1").arg( StrX(e.getMessage()).localForm() ) ) );
	} catch ( const DOMException&  e2 ) {
		QString message;
		const unsigned int maxChars = 2047;
		XMLCh errText[maxChars + 1];

		if (DOMImplementation::loadDOMExceptionMsg( e2.code, errText, maxChars ) ) {
			message = QString( "\nDOM Error during parsing\n"
						  "DomException code is: %1\n"
						  "Message is %2" ).arg( e2.code ).arg( StrX(errText).localForm() );
		} else {
			message = QString( "\nDOM Error during parsing\n"
						  "DomException code is: %1\n" ).arg( e2.code );
		}


	} catch (...) {
		QFAIL( "\nUnexpected exception during parsing\n" );
	}

}
